from typing import Optional, Iterable, List
from collections import namedtuple

import pyexcel
from xlrd.biffh import XLRDError

from .report import XlsSheetReport, XlsReport
from .xls_formats import XlsParserFormat


XlsParseResult = namedtuple(
    'Point', [
        'last_name',
        'document_series',
        'document_number',
        'date_of_birth',
        'entity_name',
        'entity_inn',
    ])


class XlsParser:
    xls_path: str
    xls_format: XlsParserFormat

    def __init__(self, xls_path: str, xls_format: XlsParserFormat):
        self.xls_path = xls_path
        self.xls_format = xls_format

    def process_row_v1(self, row) -> Optional[XlsParseResult]:
        # Обработка строки, где указаны фамилия и год рождения
        if len(row) < 7:
            print(f"ФАЙЛ НЕ СООТВЕСТВУЕТ ФОРМАТУ {XlsParserFormat.first_format.value} - {self.xls_path}, СТРОКА: {row}")
            return None
        last_name = row[1]
        document_series = str(row[2])
        document_number = str(row[3])
        date_of_birth = row[4]
        try:
            entity_name = row[5].replace('\r\n', '').strip()
        except IndexError:
            print(f"ВНИМАНИЕ! {self.xls_path} - не указано наименование организации ")
            entity_name = f"ОШИБКА НЕ УКАЗАНО НАЗВАНИЕ ОРГАНИЗАЦИИ {self.xls_path}"
        try:
            entity_inn = row[6]
        except IndexError:
            print(f"ВНИМАНИЕ! {self.xls_path} - не указан ИНН организации ")
            entity_inn = f"ОШИБКА НЕ УКАЗАН ИНН ОРГАНИЗАЦИИ {self.xls_path}"
        return self.create_result(last_name=last_name, document_series=document_series, document_number=document_number,
                                  date_of_birth=date_of_birth, entity_name=entity_name, entity_inn=entity_inn)

    def process_row_v2(self, row) -> Optional[XlsParseResult]:
        # Обработка строки, где не указаны фамилия и год рождения
        if len(row) < 5:
            print(f"ФАЙЛ НЕ СООТВЕСТВУЕТ ФОРМАТУ {XlsParserFormat.second_format.value} - {self.xls_path}, СТРОКА: {row}")
            return None
        document_series = str(row[1])
        document_number = str(row[2])
        try:
            entity_name = row[3].replace('\r\n', '').strip()
        except IndexError:
            print(f"ВНИМАНИЕ! {self.xls_path} - не указано наименование организации ")
            entity_name = f"ОШИБКА НЕ УКАЗАНО НАЗВАНИЕ ОРГАНИЗАЦИИ {self.xls_path}"
        try:
            entity_inn = row[4]
        except IndexError:
            print(f"ВНИМАНИЕ! {self.xls_path} - не указан ИНН организации ")
            entity_inn = f"ОШИБКА НЕ УКАЗАН ИНН ОРГАНИЗАЦИИ {self.xls_path}"
        return self.create_result(document_series=document_series, document_number=document_number,
                                  entity_name=entity_name, entity_inn=entity_inn)

    def create_result(self, **params):
        # Обязательный параметры
        document_series = params['document_series']
        document_number = params['document_number']
        entity_name = params['entity_name']
        entity_inn = params['entity_inn']
        # Необязательные параметры
        last_name = params.get('last_name')
        date_of_birth = params.get('date_of_birth')
        # удаляем шапку
        if 'серия' in document_series.lower() and 'номер' in document_number.lower():
            return None
        # Удаляет пробелы и табуляции в колонках C и D
        document_series = document_series.replace(' ', '').replace('\t', '').strip()
        document_number = document_number.replace(' ', '').replace('\t', '').strip()
        # проверяет что в столбце C все данные 4 символа, колонке D - 6 символов, в случае если цифр меньше, заполняет недостающие нулями спереди ('0 или '00 или '000)
        document_series = document_series.rjust(4, '0')
        document_number = document_number.rjust(6, '0')
        # В колонке F заменяет все /r/n на пробел
        entity_name = entity_name.replace('\r\n', '').strip()
        return XlsParseResult(
            last_name=last_name,
            document_series=document_series,
            document_number=document_number,
            date_of_birth=date_of_birth,
            entity_name=entity_name,
            entity_inn=entity_inn,
        )

    def process_row(self, row) -> Optional[XlsParseResult]:
        if self.xls_format == XlsParserFormat.first_format:
            return self.process_row_v1(row)
        elif self.xls_format == XlsParserFormat.second_format:
            return self.process_row_v2(row)
        else:
            raise Exception(f"Неизвестный формат файла: {self.xls_format}")

    def parse(self, xls_reports_list: List[XlsReport] = None) -> Iterable[Optional[XlsParseResult]]:
        sheet_reports = []
        xls_report = XlsReport(self.xls_path, self.xls_format, sheet_reports)
        xls_reports_list.append(xls_report)
        try:
            book = pyexcel.get_book(file_name=self.xls_path)
        except XLRDError:
            xls_report.error_msg = f"Ошибка парсинга файла {self.xls_path}"
            print(xls_report.error_msg)
            return
        sheet_numbers = book.number_of_sheets()
        for sheet_number in range(sheet_numbers):
            sheet = book.sheet_by_index(sheet_number)
            report = XlsSheetReport(
                self.xls_path,
                self.xls_format,
                sheet.name,
                sheet_number,
                sheet.number_of_rows(),
            )
            for row in sheet:
                result_row = self.process_row(row)
                if result_row is None:
                    report.rows_ignored += 1
                    continue
                report.rows_processed += 1
                yield result_row
            sheet_reports.append(report)


import os
from pyexcel_xlsx import save_data
from typing import Iterable, Tuple, List

from . import consts
from . import arg_parse
from . import exceptions
from . import report
from xlsx_parser.xls_finder import XlsFinder
from xlsx_parser.xls_parser import XlsParser, XlsParseResult, XlsParserFormat


def get_parsed_rows(xls_dir_path, xls_format: XlsParserFormat, xls_reports_list: List[report.XlsReport]):
    for xls_file in XlsFinder(xls_dir_path).get_files():
        for row in XlsParser(xls_file, xls_format).parse(xls_reports_list=xls_reports_list):
            yield row


def get_counted_rows(rows: Iterable[XlsParseResult]) -> Iterable[Tuple]:
    counter = 0
    for row in rows:
        counter += 1
        yield (counter,) + row


def save_rows(output_file, rows: Iterable[XlsParseResult]):
    counted_rows = get_counted_rows(rows)
    save_data(output_file, counted_rows)
    print(f"Данные сохранены в файл: {output_file}")


def save_reports(report_file: str, xls_reports_list: List[report.XlsReport]):
    for_write = ""
    for rep in xls_reports_list:
        for_write += str(rep) + "\n\n"
    with open(report_file, 'w') as wf:
        wf.write(for_write)
    print(for_write)
    print(f"Отчет сохранен в файл: {report_file}")


def parse(xls_dir: str, xls_format: XlsParserFormat, result_file: str, report_file: str):
    if os.path.exists(result_file):
        os.remove(result_file)
    if os.path.exists(report_file):
        os.remove(report_file)
    try:
        xls_reports_list = []
        rows = get_parsed_rows(xls_dir, xls_format, xls_reports_list)
        save_rows(result_file, rows)
        save_reports(report_file, xls_reports_list)
    except exceptions.XlsParseException as exc:
        print(exc.message)
        return


def run():
    xls_dir, output_dir, xls_format = arg_parse.parse()
    current_dir = os.path.abspath(os.path.normpath(os.getcwd()))
    if xls_dir is None:
        xls_dir = current_dir
    if output_dir is None:
        output_dir = os.path.join(current_dir, consts.default_result_dir_name)
    if not os.path.exists(output_dir):
        os.mkdir(output_dir)
    output_result_file_full_path = os.path.join(output_dir, consts.output_result_file_name)
    output_report_file_full_path = os.path.join(output_dir, consts.output_report_file_name)
    parse(xls_dir, xls_format, output_result_file_full_path, output_report_file_full_path)



import glob
from typing import Generator

from . import consts


class XlsFinder:
    source_dir: str

    files_extensions = (
        '.xlsx',
        '.xls',
    )

    def __init__(self, source_dir: str):
        self.source_dir = source_dir

    def get_files(self) -> Generator[str, None, None]:
        for ext in self.files_extensions:
            for path in glob.iglob(self.source_dir + f'/**/*{ext}', recursive=True):
                if consts.output_result_file_name in path:
                    continue
                yield path



import os

from xlsx_parser import bootstrap
from xlsx_parser.xls_parser import XlsParserFormat

current_dir = os.path.abspath(os.path.normpath(os.path.dirname(__file__)))


def test_format_v1():
    samples_v1_dir = os.path.join(current_dir, 'input', 'samples_format_1')
    output_v1_file = os.path.join(current_dir, 'output', 'format_v1', 'results.xlsx')
    output_v1_report = os.path.join(current_dir, 'output', 'format_v1', 'report.txt')
    bootstrap.parse(samples_v1_dir, XlsParserFormat.first_format, output_v1_file, output_v1_report)


def test_format_v2():
    samples_v2_dir = os.path.join(current_dir, 'input', 'samples_format_2')
    output_v2_file = os.path.join(current_dir, 'output', 'format_v2', 'results.xlsx')
    output_v2_report = os.path.join(current_dir, 'output', 'format_v2', 'report.txt')
    bootstrap.parse(samples_v2_dir, XlsParserFormat.second_format, output_v2_file, output_v2_report)


def run():
    # test_format_v1()
    test_format_v2()


if __name__ == '__main__':
    run()



from typing import Tuple

import argparse

from .xls_formats import XlsParserFormat


def parse() -> Tuple[str, str, XlsParserFormat]:
    parser = argparse.ArgumentParser(
        description="Парсер xls/xlsx-файлов для загрузки в реестр",
    )
    parser.add_argument("--folder", "-f", type=str, help="Директория с xls/xlsx-файлами",
                        required=False, default=None)
    parser.add_argument("--output", "-o", type=str,
                        help="Путь к директории с выводом",
                        required=False, default=None)
    parser.add_argument("--format", type=int,
                        help="Формат данных",
                        required=False, default=XlsParserFormat.first_format.value)
    options = parser.parse_args()
    return options.folder, options.output, XlsParserFormat(options.format)



from setuptools import setup, find_packages

import xlsx_parser

setup(
    name='xlsx_parser',
    version=xlsx_parser.__version__,
    author='someDeveloper',
    author_email='someDeveloper@someHuman.com',
    packages=find_packages(),
    package_dir={'xlsx_parser': 'xlsx_parser'},
    install_requires=[
        'pyexcel==0.5.15',
        'pyexcel-xls==0.5.8',
        'pyexcel-xlsx==0.5.8',
        'pyexcel-xlsxr==0.5.2',
        'pyexcel-xlsxw==0.4.2',
        'xlrd==1.2.0'
    ],
)


from collections import Iterable
from typing import Optional, List

from .xls_formats import XlsParserFormat


class XlsSheetReport:
    # Заполняются при инициализации
    file_path: str
    xls_format: XlsParserFormat
    sheet_name: str
    sheet_order: int
    rows_cnt: int
    # Заполняются после инициализации
    rows_processed: int
    rows_ignored: int

    def __init__(self, file_path, xls_format, sheet_name, sheet_order, rows_cnt):
        self.file_path = file_path
        self.xls_format = xls_format
        self.sheet_name = sheet_name
        self.sheet_order = sheet_order
        self.rows_cnt = rows_cnt
        self.rows_processed = 0
        self.rows_ignored = 0

    def __str__(self):
        return f"\tНазвание листа: {self.sheet_name} \n" \
               f"\tНомер листа: {self.sheet_order}\n" \
               f"\tСтрок обработано: {self.rows_processed}\n" \
               f"\tСтрок проигнорировано: {self.rows_ignored}\n" \
               f"\tСтрок всего: {self.rows_cnt}"


class XlsReport:
    file_path: str
    xls_format: XlsParserFormat
    sheet_reports: List[XlsSheetReport]
    error_msg: Optional[str]

    def __init__(self, file_path: str, xls_format: XlsParserFormat, sheet_reports: List[XlsSheetReport]):
        self.file_path = file_path
        self.xls_format = xls_format
        self.sheet_reports = sheet_reports
        self.error_msg = None

    def __str__(self):
        s = f"Имя: {self.file_path}\n" \
        f"Формат: {self.xls_format.value}\n"
        if self.error_msg:
            s += f"Ошибка: {self.error_msg if self.error_msg else ''}\n"
        s += f"Строки:\n" + '\n\t########\n'.join(map(str, self.sheet_reports))
        return s